Module core

Module core 

Structs

AtomicMatrix
The structural core of the matrix.
BlockHeader
Header structure that is written at the beginning of each block/sector
RelativePtr
A Relative Pointer to the block memory address, relative to the start of the matrix inside the process memory scope.