#[repr(C, align(16))]pub struct BlockHeader {
pub size: AtomicU32,
pub state: AtomicU32,
pub prev_phys: AtomicU32,
pub next_free: AtomicU32,
pub prev_free: AtomicU32,
}Expand description
Header structure that is written at the beginning of each block/sector
The block is made entirely of atomic primitives to ensure safe reading and manipulation across participant modules in the matrix.
Fields
size: AtomicU32state: AtomicU32prev_phys: AtomicU32next_free: AtomicU32prev_free: AtomicU32Trait Implementations