RelativePtr

Struct RelativePtr 

pub struct RelativePtr<T> { /* private fields */ }
A Relative Pointer to the block memory address, relative to the start of the matrix inside the process memory scope.

This RelativePointer is used to calculate the accurate address of the block its related to. Providing a way for independent process to localize the data inside their own mappings of the SHM segment.

It also receives a PhantomData to inform the compiler we safely own whatever generic type the caller has passed to this pointer.

Implementations

Source

impl<T> RelativePtr<T>

pub fn new(offset: u32) -> Self

Creates a new relative pointer based on the provided offset

This initializes the pointer with the PhantomData ownership over the type we are passing the the parameter

Params:

@offset: The offset value to be wrapped in the pointer.

Returns:

A instance of Self.

pub fn offset(&self) -> u32

Returns the offset value in the pointer

pub unsafe fn resolve_header<'a>(&self, base_ptr: *const u8) -> &'a BlockHeader

Resolves the header based on the base_ptr of the current caller process.

This ensures that the pointer returned is actually mapped to the process local memory scope

Params:

@base_ptr: The offset from the start of the SHM segment.

Returns:

A life time speficied reference to the header of this block

pub unsafe fn resolve_header_mut<'a>( &self, base_ptr: *const u8, ) -> &'a mut BlockHeader

pub unsafe fn resolve<'a>(&self, base_ptr: *const u8) -> &'a T

Resolves the block scope based on the base_ptr of the current caller process.

This ensures that the pointer returned is actually mapped to the process local memory scope

Params:

@base_ptr: The offset from the start of the SHM segment.

Returns:

A life time specified reference to the block scope.

pub unsafe fn resolve_mut<'a>(&self, base_ptr: *const u8) -> &'a mut T

Resolves the block scope based on the base_ptr of the current caller process.

This ensures that the pointer returned is actually mapped to the process local memory scope

Params:

@base_ptr: The offset from the start of the SHM segment.

Returns:

A life time specified mutable reference to the block scope.