Structs
- Mapping
- A helper struct that provided the O(1) calculations to find the coordinates of a block that suits exactly the requested buffer size, or the next available one that can fit the message as well.
Constants
- HEADER_
SPACE - STATE_
ACKED - STATE_
ALLOCATED - STATE_
COALESCING - STATE_
FREE - Header lifecycle flags
- SYS_
FORMATTING - SYS_
READY - SYS_
UNINITIALIZED - System initialization flags