pub struct LoopWindow<'a> { /* private fields */ }Expand description
The view object that is returned as Iterate during operations.
It provides operation abstractions on the RelativePtr and the
MatrixHandler objects related to the context of iterating over the
matrix.
Implementations
Sourceimpl<'a> LoopWindow<'a>
impl<'a> LoopWindow<'a>
Sourcepub fn new(rel_ptr: RelativePtr<u8>, handler: &'a MatrixHandler) -> Self
pub fn new(rel_ptr: RelativePtr<u8>, handler: &'a MatrixHandler) -> Self
Creates a new LoopWindow instance of a record.
Params:
@rel_ptr: The RelativePtr of the current record.
@handler: A reference to the handler owned by the caller.
Returns:
An instance of Self.
Sourcepub fn view_data_as<T>(&self) -> &'a T
pub fn view_data_as<T>(&self) -> &'a T
Returns a reference to the current record data typed as T.
Sourcepub fn view_data_raw(&self) -> &'a [u8] ⓘ
pub fn view_data_raw(&self) -> &'a [u8] ⓘ
Returns a reference to the current record data as a raw pointer.
Sourcepub fn view_header(&self) -> &'a BlockHeader
pub fn view_header(&self) -> &'a BlockHeader
Returns a reference to the header of this record
Sourcepub fn view_offset(&self) -> u32
pub fn view_offset(&self) -> u32
Returns the offset of the current record