Module core

Module core 

Structs

AtomicMatrix
The structural core of the matrix.
BlockHeader
Header structure that is written at the beginning of each block/sector
Mapping
A helper struct that provided the O(1) calculations to find the coordinates of a block that suits exactly the requested buffer size, or the next available one that can fit the message as well.
RelativePtr
A Relative Pointer to the block memory address, relative to the start of the matrix inside the process memory scope.