Module core
atomic_
matrix
0.1.0
Module core
Module Items
Structs
In atomic_
matrix::
matrix
atomic_matrix
::
matrix
Module
core
Structs

Atomic
Matrix
The structural core of the matrix.
Block
Header
Header structure that is written at the beginning of each block/sector
Mapping
A helper struct that provided the O(1) calculations to find the coordinates of a block that suits exactly the requested buffer size, or the next available one that can fit the message as well.
Relative
Ptr
A Relative Pointer to the block memory address, relative to the start of the matrix inside the process memory scope.