Module helpers

Module helpers 

Structs

Mapping
A helper struct that provided the O(1) calculations to find the coordinates of a block that suits exactly the requested buffer size, or the next available one that can fit the message as well.

Constants

STATE_ACKED
STATE_ALLOCATED
STATE_COALESCING
STATE_FREE
Header lifecycle flags
SYS_FORMATTING
SYS_READY
SYS_UNINITIALIZED
System initialization flags